package cc.smtweb.system.bpm.web.sys.user.menuPlan;

import cc.smtweb.framework.core.db.DbEngine;
import cc.smtweb.framework.core.exception.BizException;
import cc.smtweb.framework.core.util.JsonUtil;

import java.util.ArrayList;
import java.util.List;

/**
 * @Author: tanghp
 * @Date: 2022-08-02 13:16
 * @Desc: 菜单方案content字段处理类
 */
public class MenuPlanContent {
    private List<MenuPlanItem> list = new ArrayList<>();
    public MenuPlanContent(String content){
      List<MenuPlanItem> tempList = JsonUtil.parseList(content,MenuPlanItem.class);
      if(tempList!=null){
          list = tempList;
      }
    }
    // 保存节点
    public void saveItem(MenuPlanItem item,long parentId){
        boolean isNew = item.getId() <=0;
        if(isNew){
            item.setId(DbEngine.getInstance().nextId());
            if(parentId<=0){
                list.add(item);
            }else {
                MenuPlanItem parentItem = findById(parentId);
                if(parentItem==null){
                    throw new BizException("上级节点无效");
                }
                parentItem.getChildren().add(item);
            }
        }else {
            MenuPlanItem oldItem = findById(item.getId());
            if(oldItem==null){
                throw new BizException("未找到修改对象");
            }
            // 判断是否修改了父节点
            MenuPlanItem oldParent = findParentById(item.getId());
            MenuPlanItem newParent = findById(parentId);
            boolean changeParent = (oldParent ==null? -1 : oldParent.getId()) != parentId;
            if(changeParent){
                // 改了父节点 父节点的children需要移除，新的父节点需要添加
                removeById(item.getId(),oldParent==null ? list: oldParent.getChildren());
                addItem(item,newParent == null ? list : newParent.getChildren());
            }else {
                // 没有改变父节点 直接拷贝值
                oldItem.setLabel(item.getLabel());
                oldItem.setMenu(item.getMenu());
                oldItem.setIcon(item.getIcon());
                oldItem.setSeq(item.getSeq());
            }

        }
    }
    // 转换成字符串
    public String toContentStr(){
        return JsonUtil.encodeString(list);
    }
    // 根据id找节点
    public MenuPlanItem findById(long id){
        if(id<=0){
            return null;
        }
        return findById(id,list);
    }
    // 根据id找父节点
    public MenuPlanItem findParentById(long id){
        if(id<=0){
            return null;
        }
        return findParentById(id,list);
    }
    // 获取子节点
    public List<MenuPlanItem> getChildren(long parentId){
        if(parentId<=0){
            return list;
        }
        MenuPlanItem parent = findParentById(parentId);
        return parent == null ? new ArrayList<>(): parent.getChildren() ;
    }
    private MenuPlanItem findById(long id,List<MenuPlanItem> itemList){
        for(MenuPlanItem item: itemList){
            if(item.getId() == id){
                return  item;
            }
            if(item.getChildren().size()>0){
                return findById(id,item.getChildren());
            }
        }
        return null;
    }
    private MenuPlanItem findParentById(long id,List<MenuPlanItem> itemList){
        for(MenuPlanItem item: itemList){
            if(item.getChildren().size()>0){
              List<MenuPlanItem> children = item.getChildren();
              for(MenuPlanItem i : children){
                  if(i.getId() == id){
                    return  item;
                  }
                  if(i.getChildren().size()>0){
                      return findParentById(id,i.getChildren());
                  }
              }
            }
        }
        return null;
    }
    private void removeById(long id, List<MenuPlanItem> itemList){
        if(itemList == null || itemList.size()<=0){
            return;
        }
        itemList.removeIf(item -> item.getId() == id);
    }
    private void addItem(MenuPlanItem item, List<MenuPlanItem> itemList){
        if(itemList == null){
            return;
        }
        itemList.add(item);
    }
}
