package cc.smtweb.system.bpm.web.sys.user.menuPlan;

import cc.smtweb.framework.core.common.R;
import cc.smtweb.framework.core.common.SwMap;
import cc.smtweb.framework.core.db.DbEngine;
import cc.smtweb.framework.core.db.jdbc.AbsDbWorker;
import cc.smtweb.framework.core.exception.BizException;
import cc.smtweb.framework.core.mvc.service.AbstractHandler;
import cc.smtweb.framework.core.mvc.service.SwListData;
import cc.smtweb.framework.core.util.JsonUtil;
import cc.smtweb.system.bpm.web.design.form.define.PageDataset;
import cc.smtweb.system.bpm.web.engine.dynPage.DynPageListHandler;
import cc.smtweb.system.bpm.web.engine.dynPage.DynPageLoadHandler;
import cc.smtweb.system.bpm.web.engine.dynPage.DynPageProvider;
import cc.smtweb.system.bpm.web.engine.dynPage.DynPageTreeHandler;

import java.util.ArrayList;
import java.util.List;

/**
 * @Author: tanghp
 * @Date: 2022-08-01 18:10
 * @Desc:
 */
public class MenuPlanHandler {
    // 菜单列表数据集
    private static String DS_MENU_LIST = "menuList";
    private static String DS_MENU_CARD = "menuCard";
    private static String DS_MENU_TREE = "menuTree";
    static class MenuCustHandler extends AbstractHandler {
        // 保存方案明细
       public R savePlanItem(){
           // 方案ID
           long plan_id = params.readLong("plan_id");
           if(plan_id<=0){
               throw new BizException("所属方案不能为空！");
           }
          MenuPlan menuPlan = MenuPlanCache.getInstance().get(plan_id);
           if(menuPlan==null){
               throw new BizException("所属方案不能为空！");
           }
           long parent_id = params.readLong("parent_id",-1L);
           MenuPlanItem menuPlanItem = JsonUtil.parse(params, MenuPlanItem.class);
           // 处理content大字段
           MenuPlanContent menuPlanContent = new MenuPlanContent(menuPlan.getContent());
           menuPlanContent.saveItem(menuPlanItem,parent_id);
           menuPlan.setContent(menuPlanContent.toContentStr());

           DbEngine.getInstance().doTrans(new AbsDbWorker() {
               @Override
               public void work() {
                   DbEngine.getInstance().updateEntity(menuPlan,"smp_content");
               }

               @Override
               public void doAfterDbCommit() {
                   MenuPlanCache.getInstance().put(menuPlan);
               }
           });
           return R.success(menuPlanItem);
       }
    }

    static class MenuLoadHandler extends DynPageLoadHandler{
        @Override
        public DynPageProvider getProvider() {
            return new MenuProvider();
        }

        public DynPageListHandler getListWorker(SwMap filter, PageDataset pageDataSet) {
            if(DS_MENU_LIST.equals(pageDataSet.name)){
                return new MenuListHandler(pageId,filter,pageDataSet);
            }
            DynPageListHandler listHandler = new DynPageListHandler(pageId, filter, pageDataSet);
            listHandler.init(params, us);
            return listHandler;
        }

        @Override
        public DynPageTreeHandler getTreeWorker(SwMap filter, PageDataset pageDataSet) {
            menuTreeHandler  treeHandler = new MenuPlanHandler.menuTreeHandler(pageId,filter,pageDataSet);
            treeHandler.init(params, us);
            return treeHandler;
        }
    }
    static class MenuListHandler extends DynPageListHandler {
        public MenuListHandler(long pageId, SwMap filter, PageDataset pageDataSet) {
            super(pageId, filter, pageDataSet);
        }

        @Override
        public SwListData buildListData() {
            List<MenuPlanItem> children = getChildren();
            List<SwMap> list = new ArrayList<>();
            int total = children.size();
            for(MenuPlanItem bean: children){
                list.add(bean.getData());
            }
            return SwListData.create(list,total);
        }
        @Override
        public R getTotal() {
            SwMap r = new SwMap();
            List<MenuPlanItem> children = getChildren();
            r.put("total", children.size());
            return R.success(r);
        }
        private List<MenuPlanItem> getChildren(){
            long plan_id = filter.readLong("plan_id",-1L);
            long parent_id = filter.readLong("parent_id",-1L);
            if(plan_id<=0){
                return new ArrayList<>();
            }
            MenuPlan  menuPlan = MenuPlanCache.getInstance().get(plan_id);
            if(menuPlan==null){
                return new ArrayList<>();
            }
            MenuPlanContent menuPlanContent = new MenuPlanContent(menuPlan.getContent());
            return menuPlanContent.getChildren(parent_id);
        }
    }
    static class MenuProvider extends DynPageProvider{
        @Override
        public SwMap loadData(SwMap filter, PageDataset pageDataSet) {
            if(DS_MENU_CARD.equals(pageDataSet.name)){
                long id = filter.readLong("id",1L);
                long plan_id = filter.readLong("plan_id",1L);
                MenuPlan menuPlan = MenuPlanCache.getInstance().get(plan_id);
                if(menuPlan==null) {
                    return new SwMap();
                }
                MenuPlanContent menuPlanContent = new MenuPlanContent(menuPlan.getContent());
                MenuPlanItem menuPlanItem = menuPlanContent.findById(id);
                if(menuPlanItem == null){
                    return new SwMap();
                }
                SwMap r = new SwMap();
                r.putAll(menuPlanItem.getData());
                return r;
            }
            return super.loadData(filter, pageDataSet);
        }
    }
    static class menuTreeHandler extends DynPageTreeHandler{
        private MenuPlanContent menuPlanContent;
        public menuTreeHandler(long pageId, SwMap filter, PageDataset pageDataSet) {
            super(pageId, filter, pageDataSet);
            long plan_id = filter.readLong("plan_id",-1L);
            if(plan_id<=0){
                menuPlanContent = new MenuPlanContent(null);
            }
            MenuPlan menuPlan = MenuPlanCache.getInstance().get(plan_id);
            menuPlanContent = new MenuPlanContent(menuPlan == null ? null: menuPlan.getContent());
        }

        @Override
        protected List<SwMap> getChildren(long id) {
            List<SwMap> list = new ArrayList<>();
            List<MenuPlanItem> temList = menuPlanContent.getChildren(id);
            temList.forEach(bean-> list.add(bean.getData()));
            return list;
        }

        @Override
        protected long getId(SwMap bean) {
            return bean.readLong("id");
        }

        @Override
        protected String getText(SwMap bean) {
            return bean.readString("label");
        }
    }

}
